'use strict';

var react = require('react');
var useResolver = require('./useResolver.js');

async function serialize(element) {
    var _a, _b;
    if (!(element === null || element === void 0 ? void 0 : element.type))
        return element;
    let result = { type: element.type, props: { ...element.props } };
    if (typeof element.key === 'function') {
        return serialize(await element.key(element.props));
    }
    if (result.props.children) {
        result.props.children = await Promise.all([]
            .concat((_b = (_a = element.props) === null || _a === void 0 ? void 0 : _a.children) !== null && _b !== void 0 ? _b : [])
            .map((child) => serialize(child)));
    }
    return result;
}
function deserialize(element) {
    var _a, _b;
    if (!(element === null || element === void 0 ? void 0 : element.type))
        return element;
    let result = { type: element.type, props: { ...element.props } };
    if (result.props.children) {
        result.props.children = []
            .concat((_b = (_a = element.props) === null || _a === void 0 ? void 0 : _a.children) !== null && _b !== void 0 ? _b : [])
            .map((child) => deserialize(child));
    }
    return react.createElement(result.type, result.props);
}
function useValue(v) {
    const [state, setState] = react.useState(null);
    react.useEffect(() => {
        if (typeof v === 'function') {
            v().then(setState);
        }
        else {
            setState(v);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    return state;
}
function ServerComponent({ value }) {
    const result = useValue(value);
    const [element] = useResolver.useResolver(result);
    return deserialize(element) || null;
}

exports.ServerComponent = ServerComponent;
exports.deserialize = deserialize;
exports.serialize = serialize;
