'use strict';

var react = require('react');
var core = require('@convey/core');

function useResolver(str) {
    const structure = core.getStructure(str);
    const { resolver } = structure !== null && structure !== void 0 ? structure : {};
    const [resultPromise, setResultPromise] = react.useState(null);
    const [meta, setMeta] = react.useState({
        status: 'UNSET',
        refresh: async () => {
            const newResultPromise = new core._Promise();
            setResultPromise(newResultPromise);
            return newResultPromise;
        },
    });
    const [data, setData] = react.useState(undefined);
    react.useEffect(() => {
        if (!resolver)
            return;
        let iter;
        let finished = false;
        async function main() {
            setMeta((meta) => ({ ...meta, status: 'PENDING' }));
            if (structure.stream) {
                if (typeof resolver === 'function') {
                    iter = core.resolveStream(structure);
                }
                else {
                    iter = core.subscribeStream(structure);
                }
            }
            else {
                iter = core.subscribe(structure);
            }
            try {
                for await (let value of iter) {
                    if (finished)
                        return;
                    setData(() => value);
                    setMeta((meta) => ({ ...meta, status: 'DONE' }));
                    resultPromise === null || resultPromise === void 0 ? void 0 : resultPromise.resolve(value);
                }
            }
            catch (error) {
                setMeta((meta) => ({ ...meta, status: 'ERROR', error }));
                resultPromise === null || resultPromise === void 0 ? void 0 : resultPromise.reject(error);
            }
        }
        main();
        return () => {
            finished = true;
            iter.return();
        };
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [resolver, core.config.getResolverHash(structure), resultPromise]);
    return [data, meta];
}

exports.useResolver = useResolver;
